@@ -15,6 +15,7 @@ if gevent_socketio_found:
 
 import socketio
 import flask
+from flask import json as flask_json
 from werkzeug.debug import DebuggedApplication
 from werkzeug.serving import run_with_reloader
 
@@ -53,8 +54,10 @@ class SocketIO(object):
                     multiple clusters of SocketIO processes need to use the
                     same message queue without interfering with each other, then
                     each cluster should use a different channel.
-    :param resource: The SocketIO resource name. Defaults to ``'socket.io'``.
-                     Leave this as is unless you know what you are doing.
+    :param path: The path where the Socket.IO server is exposed. Defaults to
+                 ``'socket.io'``. Leave this as is unless you know what you are
+                 doing.
+    :param resource: Alias to ``path``.
     :param kwargs: Socket.IO and Engine.IO server options.
 
     The Socket.IO server options are detailed below:
@@ -76,7 +79,8 @@ class SocketIO(object):
     :param json: An alternative json module to use for encoding and decoding
                  packets. Custom json modules must have ``dumps`` and ``loads``
                  functions that are compatible with the standard library
-                 versions.
+                 versions. To use the same json encoder and decoder as a Flask
+                 application, use ``flask.json``.
 
     The Engine.IO server configuration supports the following settings:
 
@@ -138,7 +142,26 @@ class SocketIO(object):
                                     write_only=write_only)
                 self.server_options['client_manager'] = queue
 
-        resource = kwargs.pop('resource', 'socket.io')
+        if 'json' in self.server_options and \
+                self.server_options['json'] == flask_json:
+            # flask's json module is tricky to use because its output
+            # changes when it is invoked inside or outside the app context
+            # so here to prevent any ambiguities we replace it with wrappers
+            # that ensure that the app context is always present
+            class FlaskSafeJSON(object):
+                @staticmethod
+                def dumps(*args, **kwargs):
+                    with app.app_context():
+                        return flask_json.dumps(*args, **kwargs)
+
+                @staticmethod
+                def loads(*args, **kwargs):
+                    with app.app_context():
+                        return flask_json.loads(*args, **kwargs)
+
+            self.server_options['json'] = FlaskSafeJSON
+
+        resource = kwargs.pop('path', kwargs.pop('resource', 'socket.io'))
         if resource.startswith('/'):
             resource = resource[1:]
         self.server = socketio.Server(**self.server_options)
@@ -583,7 +606,10 @@ def send(message, **kwargs):
                          addressing a room, or ``False`` to send to everyone
                          but the sender.
     """
-    namespace = kwargs.get('namespace', flask.request.namespace)
+    if 'namespace' in kwargs:
+        namespace = kwargs['namespace']
+    else:
+        namespace = flask.request.namespace
     callback = kwargs.get('callback')
     broadcast = kwargs.get('broadcast')
     room = kwargs.get('room')
